# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2019-03-05 09:23
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tests', '0005_avatar'),
    ]

    operations = [
        migrations.RenameField(
            model_name='avatar',
            old_name='custormer',
            new_name='customer',
        ),
    ]
