# Generated by Django 3.1 on 2021-01-02 10:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('gdpr', '0009_migration'),
    ]

    operations = [
        migrations.AlterField(
            model_name='legalreason',
            name='purpose_slug',
            field=models.CharField(db_index=True, max_length=100, verbose_name='purpose'),
        ),
    ]
