# Generated by Django 2.1.7 on 2019-03-20 10:06

from django.db import migrations, models
import django.db.models.deletion
import gdpr.mixins


class Migration(migrations.Migration):
    dependencies = [
        ('tests', '0006_auto_20190305_0323'),
    ]

    operations = [
        migrations.CreateModel(
            name='ExtraParentD',
            fields=[
                ('id_d', models.AutoField(editable=False, primary_key=True, serialize=False)),
                ('note', models.CharField(max_length=250)),
            ],
            options={
                'abstract': False,
            },
            bases=(gdpr.mixins.AnonymizationModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='TopParentA',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=250)),
            ],
            options={
                'abstract': False,
            },
            bases=(gdpr.mixins.AnonymizationModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='ParentB',
            fields=[
                ('topparenta_ptr',
                 models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True,
                                      primary_key=True, serialize=False, to='tests.TopParentA')),
                ('birth_date', models.DateField()),
            ],
            options={
                'abstract': False,
            },
            bases=('tests.topparenta',),
        ),
        migrations.CreateModel(
            name='ParentC',
            fields=[
                ('parentb_ptr',
                 models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True,
                                      primary_key=True, serialize=False, to='tests.ParentB')),
                ('first_name', models.CharField(max_length=250)),
            ],
            options={
                'abstract': False,
            },
            bases=('tests.parentb',),
        ),
        migrations.CreateModel(
            name='ChildE',
            fields=[
                ('extraparentd_ptr',
                 models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True,
                                      to='tests.ExtraParentD')),
                ('parentc_ptr',
                 models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True,
                                      primary_key=True, serialize=False, to='tests.ParentC')),
                ('last_name', models.CharField(max_length=250)),
            ],
            options={
                'abstract': False,
            },
            bases=('tests.parentc', 'tests.extraparentd'),
        ),
    ]
