# Generated by Django 2.1.5 on 2019-02-08 17:48

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('tests', '0002_note'),
    ]

    operations = [
        migrations.AddField(
            model_name='account',
            name='IBAN',
            field=models.CharField(blank=True, max_length=34, null=True),
        ),
        migrations.AddField(
            model_name='account',
            name='swift',
            field=models.CharField(blank=True, max_length=11, null=True),
        ),
    ]
