# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2019-02-25 16:27
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('tests', '0003_IBAN'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='customer',
            name='fb_id',
        ),
        migrations.AddField(
            model_name='customer',
            name='facebook_id',
            field=models.CharField(blank=True, help_text='Facebook ID used for login via Facebook.', max_length=256,
                                   null=True, verbose_name='Facebook ID'),
        ),
    ]
