# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2019-02-25 16:27
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('gdpr', '0004'),
    ]

    operations = [
        migrations.AlterField(
            model_name='anonymizeddata',
            name='content_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, to='contenttypes.ContentType',
                                    verbose_name='related object content type'),
        ),
        migrations.AlterField(
            model_name='anonymizeddata',
            name='expired_reason',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL,
                                    to='gdpr.LegalReason', verbose_name='expired reason'),
        ),
        migrations.AlterField(
            model_name='legalreason',
            name='expires_at',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='expires at'),
        ),
        migrations.AlterField(
            model_name='legalreason',
            name='source_object_content_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, to='contenttypes.ContentType',
                                    verbose_name='source object content type'),
        ),
        migrations.AlterField(
            model_name='legalreason',
            name='source_object_id',
            field=models.TextField(verbose_name='source object ID'),
        ),
        migrations.AlterField(
            model_name='legalreasonrelatedobject',
            name='object_content_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, to='contenttypes.ContentType',
                                    verbose_name='related object content type'),
        ),
        migrations.AlterField(
            model_name='legalreasonrelatedobject',
            name='object_id',
            field=models.TextField(verbose_name='related object ID'),
        ),
        migrations.AlterUniqueTogether(
            name='anonymizeddata',
            unique_together=set([('content_type', 'object_id', 'field')]),
        ),
        migrations.AlterUniqueTogether(
            name='legalreason',
            unique_together=set([('purpose_slug', 'source_object_content_type', 'source_object_id')]),
        ),
        migrations.AlterUniqueTogether(
            name='legalreasonrelatedobject',
            unique_together=set([('legal_reason', 'object_content_type', 'object_id')]),
        ),
    ]
