# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2018-12-19 15:17
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('gdpr', '0003'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='legalreason',
            unique_together=set([('purpose_slug', 'source_object_content_type', 'source_object_id')]),
        ),
        migrations.AlterUniqueTogether(
            name='legalreasonrelatedobject',
            unique_together=set([('legal_reason', 'object_content_type', 'object_id')]),
        ),
    ]
