# Generated by Django 3.1 on 2021-04-18 10:56

from django.db import migrations
import enumfields.fields
import gdpr.enums


class Migration(migrations.Migration):

    dependencies = [
        ('gdpr', '0010_migration'),
    ]

    operations = [
        migrations.AlterField(
            model_name='legalreason',
            name='state',
            field=enumfields.fields.NumEnumField(
                db_index=True, default=1, enum=gdpr.enums.LegalReasonState, verbose_name='state'
            ),
        ),
    ]
