# -*- coding: utf-8 -*-
# Generated by Django 1.11.26 on 2020-02-17 13:26
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('gdpr', '0008_migration'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='legalreason',
            name='is_active',
        ),
    ]
